/*  
 By Doug Coulter for real data aquisition and a useful output format.
 Begun 12/4/2013
 Counter 1 (hardware counter) added 12/7/13  input is arudino pin 5, 
 falling edge, with pullup.  Prints out and is labeled as C0.
 Counter 2 (labeled C1) is the int0 input, added 12/18/13

GPL V2 license - share and enjoy, but if you come up with
a cool new trick, pleasse send it to dcfusor@gmail.com so I can
put it up on the forums for all of us to share and enjoy.
(www.coultersmithing.com/forums)
*/

/*  NOTE!  What I call counter 1 (hardware counter 1) and counter 2,
which counts interrupts (on ext interrupt zero), are actually 
labeled counter 0 and 1 on the hardware front panel.

Sorry, it's confusing to someone no matter how I do it.

I decided to make it simple for those who can't read code anyway.
Just know to read the front panel.  C0 bnc goees to hardware counter 1,
and C1 goes to external interrupt 1 (and really eats cycles).
So, in code, I labeled them kinda like the hardware the code is
talking to.  In the physical box, they're labeled zero and one.
zero is the fast one - you can slam that one up to around a few
mhz.  interrupt zero (C1 on the front panel, c2 in here) can't
take super fast counts...ISR save/restore time...kills us.
*/


//#include <EEPROM.h> // used to count boots...a waste of time for now
//#include "/home/doug/bin/i2cbitbang.c";
#include <SD.h> // someday, we'll log to SD card 3890 bytes w/o the include
// 7939 with include and zero usage of this(!)  4k bytes before calling it!

//   globals ****************************************************
#define ADCHNLS 6  // no ; after defines in arduino land
int AD[ADCHNLS];        // values read from the a/d
boolean digout = false;
unsigned long old_millis = 0;   // Z-1 for timing, rather than just delaying
unsigned long ADMillis = 0;
byte pass = 0;

volatile unsigned long c1counts;    // total counts from (hardware) counter 1,
// top half incremented by timer overflow
unsigned long c1z = 0; // previous count for subtraction
volatile unsigned long c2counts;
unsigned long c2z = 0;
unsigned long curcount =  0; // what we'll send, this - last total
unsigned long cur2count = 0; // added for interrupt based counter
//byte boots = 0;    // count total boots
/****************************************************************/

//////////////////////////////////////////////// subs called from setup
void t1set() // setup timer1 as a counter that interrupts on overflow
{
  TCCR1A = 0; // turn off all the weird compare/capture modes etc
  TCCR1B = 6; // clock on falling edge (7 for rising)
  TCNT1 = 0; //zero out timer
  TIMSK1 = _BV(TOIE1);// sets the timer overflow interrupt enable bit
  }
/////////////////////////////////////////////////////////
//////// manual isr setup, you bet, saves a whole library
// actual isr for hardware counter 1
ISR(TIMER1_OVF_vect)
{
 c1counts += 0x00010000; // add one to top half of longword
} 
////////////////////////////////////////////////
void intCounter ()
{
 c2counts++; // pretty simple... 
}
////////////////////////////////////////////////
////////////////////////////////////////////////

////////////////////////////////////////////////
/* *********************************************
 Always hit on any reset - soft, hard, serial port connect...
*/
void setup() {
  // initialize serial communications
  Serial.begin(115200); // 115,200? Yeah!
//  Serial.print("Setup reached:"); // every reset of any kind
  pinMode(13,INPUT);
  pinMode(9,OUTPUT);  // scope for loop timing etc
  pinMode(5,INPUT); // counter input @@@ INPUT_PULLUP later?
  pinMode(2,INPUT); // external interrupt 0
  //@@@ their bug - INPUT_PULLUP not recognized like dox say
  digitalWrite(5, HIGH); // turns on pullup resistor
  digitalWrite(2,HIGH); // same as above
  digitalWrite(13,LOW); // save led power on that pin
  t1set(); // setup timer 1 after ensuring high input
  attachInterrupt(0,intCounter,FALLING); // works but eats more bytes

//  boots = EEPROM.read(0);
//  boots++;
//  EEPROM.write(0,boots);
//  Serial.print(boots);
    
    c1counts = 0; // sometimes get an initial interrupt from hardware
    Serial.println("#\tSetup end"); // every reset of any kind
    
}
//******** subroutines defined before use below in loop() **********

////////////////////////////////////////////////
////////////////////////////////////////////////
////////////////////////////////////////////////
////////////////////////////////////////////////

//////////////////////////////////////////////// 
void soft_reset () // software reset, actually a hard RST
{ // so we can reset timers and so on under host control
 asm volatile ("  jmp 0"); 
} // this really is a hard reset, but coming from software
////////////////////////////////////////////////
void clearAD()
{
 byte chnl; 
  for (chnl=0; chnl < ADCHNLS; chnl++)
  {
   AD[chnl] = 0; // clear out old
  }
  pass = 0;
}

////////////////////////////////////////////////
void readAD()
{
 byte chnl;  // Analog input pin 
 
   for (chnl=0; chnl < ADCHNLS; chnl++)
   {
//    delayMicroseconds(10); // Maybe not required, it's slow enough.
    AD[chnl]  += analogRead(chnl); // delay required?
   }
 ADMillis = millis(); // so we can time for next pass through
 pass++; // so we know how many we've done 
}
////////////////////////////////////////////////
void readCounts()
{ // now gets both counters
/* Disable interrupts */
noInterrupts();
/* Read TCNT1 into c1counts low part */
c1counts &= 0xffff0000;
c1counts |= TCNT1; // from the hardware register
curcount = c1counts - c1z; // just the new ones for this time interval
c1z = c1counts; // for next time

cur2count = c2counts - c2z; // same idea as above
c2z = c2counts; // update for doing the diff next time
/* Restore global interrupt flag */
interrupts(); 
}
////////////////////////////////////////////////
void printOut()
{
 Serial.print (millis());
 byte i;
 for (i=0; i < ADCHNLS; i++)
 {
  Serial.print ("\t");  // some delimiter
  Serial.print (AD[i]);
 }
 Serial.print ("\t");
 Serial.print(curcount); // new (only) counts from hardware counter
 Serial.print ("\t");
 Serial.print (cur2count);
 Serial.println(""); // arduino windows CRLF for now (PITA for linux)
}
////////////////////////////////////////////////
void HandleHost()
{ // take care of anything coming into serial input
/*
I was going to do this as a state machine, but decided to only
allow single byte commands for now...maybe later.
About the only thing I can think of that would need multiple
bytes is setting time of day, which dev hardware doesn't have now.
*/
 byte command;
 command = Serial.read(); // get it 
 switch (command)
 {
  case '!': // restart
  soft_reset();
  break;
  
  case 'i': // identify yourself in case there's more than one of us
  Serial.println("#\tadaq1");
  break;
 } 
}


//******************************************************************
/// main loop, called repeatedly from base arduino code
void loop() {
 if (millis() - ADMillis >=6)
 {
  if (pass < 16) // not sure if this is fastest way or not
  {
   readAD();
  } 
 }
 
  if (millis() - old_millis >= 100)
 {
  old_millis += 100; // @@@ reset "fast" time-base; just add 100?
  readCounts(); // get timer 1 counts to c1counts long
  printOut(); // tell the world what we see
  clearAD();
 }
 
///* 
 if (digout) // show time between loop calls, longer time when we do work - debug
 {
  digitalWrite(9,HIGH); // on the pin
  } else
  {
   digitalWrite(9, LOW);   // off the pin 
  } 
  digout = !digout; //toggle every time through
 // */

// handle incoming commands from host PC 
  if (Serial.available() > 0) HandleHost(); // reset clock if ! sent
  
  
} // end main loop

